package moe.gensoukyo.haloshield.client.caps

import moe.gensoukyo.haloshield.SpartanShieldsMod
import moe.gensoukyo.haloshield.api.client.ShieldRenderManager
import moe.gensoukyo.haloshield.client.model.internal.GeneratedRenderers
import moe.gensoukyo.haloshield.client.renderer.RenderEnergyShield
import net.minecraft.entity.Entity
import net.minecraft.entity.EntityLivingBase
import net.minecraft.util.EnumFacing
import net.minecraft.util.ResourceLocation
import net.minecraftforge.client.event.RenderLivingEvent
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.capabilities.CapabilityInject
import net.minecraftforge.common.capabilities.ICapabilityProvider
import net.minecraftforge.event.AttachCapabilitiesEvent
import net.minecraftforge.fml.common.Mod
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent
import net.minecraftforge.fml.relauncher.Side
import net.minecraftforge.fml.relauncher.SideOnly

/**
 * @throws IllegalModelException 有的时候模型类型转换失败，此时抛出异常
 * @author ChloePrime
 */
@SideOnly(Side.CLIENT)
class CapShieldRenderer<E: EntityLivingBase>(
    entity: E
) {
    companion object {
        @CapabilityInject(CapShieldRenderer::class)
        @JvmStatic lateinit var CAP_TOKEN: Capability<CapShieldRenderer<EntityLivingBase>>
    }
    val controller = ShieldRenderManager.getController(entity)
    val renderer = GeneratedRenderers.fromEntity(entity)

}

/**
 * @author ChloePrime
 */
@SideOnly(Side.CLIENT)
class ShieldModelProvider<E: EntityLivingBase>(entity: E): ICapabilityProvider {
    private var instance = try {
        CapShieldRenderer(entity)
    } catch (e: IllegalModelException) {
        null
    } catch (e: Exception) {
        e.printStackTrace()
        null
    }

    override fun hasCapability(cap: Capability<*>, facing: EnumFacing?): Boolean {
        return cap == CapShieldRenderer.CAP_TOKEN && instance != null
    }

    @Suppress("UNCHECKED_CAST")
    override fun <T : Any> getCapability(
        cap: Capability<T>,
        facing: EnumFacing?): T? {
        return if (hasCapability(cap, facing)) (instance!! as T) else null
    }
}

/**
 * 负责添加事件以及触发渲染
 * @author ChloePrime
 */
@SideOnly(Side.CLIENT)
@Mod.EventBusSubscriber(value = [Side.CLIENT], modid = SpartanShieldsMod.MODID)
object CapShieldListener {
    private val CAP_ID = ResourceLocation(
        SpartanShieldsMod.MODID, "ex_model"
    )

    @JvmStatic
    @SubscribeEvent
    fun onCapAttach(e: AttachCapabilitiesEvent<Entity>) {
        val entity = e.`object`
        if (entity is EntityLivingBase) {
            e.addCapability(CAP_ID, ShieldModelProvider(entity))
        }
    }

    @JvmStatic
    @SubscribeEvent
    fun onPostRenderEntity(e: RenderLivingEvent.Post<EntityLivingBase>) {
        // 防止无限递归
        if (e.renderer is RenderEnergyShield<*>) {
            return
        }
        val cap = e.entity.getCapability(CapShieldRenderer.CAP_TOKEN, null) ?: return
        val renderer = cap.renderer
        renderer.tellMeTickExisted(e.entity.ticksExisted)
            .bindController(cap.controller)
            .doRender(
                e.entity, e.x, e.y, e.z, e.entity.rotationYaw, e.partialRenderTick
            )
    }
}

@SideOnly(Side.CLIENT)
class IllegalModelException: IllegalStateException()
