package moe.gensoukyo.haloshield.client.compat.cnpc

import com.github.tartaricacid.touhoulittlemaid.entity.passive.EntityMaid
import moe.gensoukyo.haloshield.ModIds
import moe.gensoukyo.haloshield.SpartanShieldsMod
import moe.gensoukyo.haloshield.api.client.IShieldRenderControl
import moe.gensoukyo.haloshield.client.compat.vanilla.SlimeShieldModel
import moe.gensoukyo.haloshield.client.event.GenerateShieldRendererEvent
import moe.gensoukyo.haloshield.client.model.EnergyShieldModel
import moe.gensoukyo.haloshield.client.model.internal.GeneratedRenderers
import moe.gensoukyo.haloshield.client.renderer.RenderEnergyShield
import moe.gensoukyo.haloshield.core.client.api.IRenderPreparationAccessor
import net.minecraft.client.model.ModelBase
import net.minecraft.client.renderer.entity.RenderLivingBase
import net.minecraft.entity.EntityLivingBase
import net.minecraftforge.fml.common.Loader
import net.minecraftforge.fml.common.Mod
import net.minecraftforge.fml.common.Optional
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent
import net.minecraftforge.fml.relauncher.Side
import net.minecraftforge.fml.relauncher.SideOnly
import noppes.npcs.client.model.ModelWrapper
import noppes.npcs.client.renderer.RenderCustomNpc
import noppes.npcs.client.renderer.RenderNPCInterface
import noppes.npcs.entity.EntityCustomNpc
import noppes.npcs.entity.EntityNPCInterface
import noppes.npcs.entity.EntityNpcSlime
import java.lang.reflect.Field

@SideOnly(Side.CLIENT)
abstract class AbstractNpcEnergyShield<E: EntityCustomNpc, R: RenderNPCInterface<E>>(
    entityClass: Class<E>,
    private val npcRenderer: R,
    shieldModel: ModelBase
): RenderEnergyShield<E>(entityClass, shieldModel) {
    /**
     * 为了兼容女仆 mod 替换后的渲染器，
     * 这两个 field 在女仆 mod 是否安装的情况下
     * 是不同的
     */
    abstract val fieldEntity: Field
    abstract val fieldRenderEntity: Field

    @Suppress("UNCHECKED_CAST")
    private val npcRenderMixin = npcRenderer
            as IRenderPreparationAccessor<EntityNPCInterface>

    private var lastEntity: EntityLivingBase? = null
    private var lastRenderer: RenderLivingBase<*>? = null
    private var cacheEnergyShieldRenderer: RenderEnergyShield<EntityLivingBase>? = null

    override fun doRender(
        entity: E,
        x: Double,
        y: Double,
        z: Double,
        entityYaw: Float,
        partialTicks: Float
    ) {
        // 这一步会刷新 NPC 引用的实体数据，
        // 需要放在检查模型是否是女仆模型之前
        tryRebuildShield()

        val isMaid = (Loader.isModLoaded(ModIds.TLM) && isMaid())
        if (isMaid) return

        super.doRender(entity, x, y, z, entityYaw, partialTicks)

        if (cacheEnergyShieldRenderer != null) {
            cacheEnergyShieldRenderer!!.rotOverride = { _, age, yaw, partial ->
                npcRenderMixin.callApplyRotations(entity, age, yaw, partial)
            }
            cacheEnergyShieldRenderer!!.preparationOverride = { _, partial ->
                npcRenderMixin.callPreRenderCallback(entity, partial)
            }
            cacheEnergyShieldRenderer!!.doRender(
                lastEntity ?: return,
                x, y, z, entityYaw, partialTicks
            )
        }
    }

    @Optional.Method(modid = ModIds.TLM)
    private fun isMaid(): Boolean {
        return lastEntity is EntityMaid
    }

    override fun applyRotations(entity: E, ageInTicks: Float, rotationYaw: Float, partialTicks: Float) {
        npcRenderMixin.callApplyRotations(entity, ageInTicks, rotationYaw, partialTicks)
        super.applyRotations(entity, ageInTicks, rotationYaw, partialTicks)
    }

    override fun preRenderCallback(entity: E, partialTick: Float) {
        npcRenderMixin.callPreRenderCallback(entity, partialTick)
        super.preRenderCallback(entity, partialTick)
    }

    override fun tellMeTickExisted(tickIn: Int): RenderEnergyShield<E> {
        cacheEnergyShieldRenderer?.tellMeTickExisted(tickIn)
        super.tellMeTickExisted(tickIn)
        return this
    }

    override fun bindController(c: IShieldRenderControl): RenderEnergyShield<E> {
        cacheEnergyShieldRenderer?.bindController(c)
        super.bindController(c)
        return this
    }


    /**
     * 当NPC的实体模型被替换时，
     * 重新构建实体模型对应的护盾模型。
     */
    private fun tryRebuildShield() {
        val curEntity = fieldEntity.get(npcRenderer) as? EntityLivingBase

        if (lastEntity != curEntity) {
            lastEntity = curEntity
            lastRenderer = fieldRenderEntity.get(npcRenderer)
                    as? RenderLivingBase<out EntityLivingBase>
            cacheEnergyShieldRenderer = lastEntity?.let { GeneratedRenderers.fromEntity(it) }
        }
    }

    /**
     * NPC 会包装模型以提供特殊材质
     * 这个方法解除 NPC 的模型包装
     */
    private fun ModelBase.getRealModel(): ModelBase {
        return if (this is ModelWrapper) this.wrapped else this
    }
}

@SideOnly(Side.CLIENT)
@Mod.EventBusSubscriber(value = [Side.CLIENT], modid = SpartanShieldsMod.MODID)
object NpcShieldListener {
    @JvmStatic
    @SubscribeEvent
    @Optional.Method(modid = ModIds.CNPC)
    fun onShieldModel(e: GenerateShieldRendererEvent<EntityCustomNpc>) {
        val renderer = e.entityRenderer as? RenderCustomNpc<EntityCustomNpc> ?: return
        e.rendererOverride = NpcEnergyShield(renderer, EnergyShieldModel(renderer.mainModel))
    }

    /**
     * 替换 NPC mod 的特殊史莱姆的护盾模型
     */
    @JvmStatic
    @SubscribeEvent
    @Optional.Method(modid = ModIds.CNPC)
    fun onNpcModel(e: GenerateShieldRendererEvent<EntityNpcSlime>) {
        e.modelOverride = SlimeShieldModel()
    }
}