package moe.gensoukyo.haloshield.client.impl

import moe.gensoukyo.haloshield.api.client.IShieldRenderControl
import moe.gensoukyo.haloshield.common.api.SpartanShieldApi
import net.minecraft.entity.EntityLivingBase
import net.minecraftforge.fml.relauncher.Side
import net.minecraftforge.fml.relauncher.SideOnly
import javax.vecmath.Color4f

@SideOnly(Side.CLIENT)
class ShieldRenderControlImpl(
    entity: EntityLivingBase
): IShieldRenderControl {
    companion object {
        /**
         * 满血显示的护盾颜色
         * 青色
         */
        private val COLOR0 = Color4f(0.35F, 0.98F, 1F, 0.66F)

        /**
         * 低血量显示的颜色
         * 红色
         */
        private val COLOR1 = Color4f(1F, 0.2F, 0.2F, 0.66F)

        private val D_COLOR = Color4f(COLOR1)
        init {
            D_COLOR.sub(COLOR0)
        }
        /**
         * getColor的返回结果缓存
         */
        private val result = Color4f(COLOR0)
        private val dColor = Color4f(D_COLOR)

        private const val gradientLength = 10
        private const val gradientSpeed = 1F / gradientLength
    }

    private val shieldData = SpartanShieldApi.getShieldData(entity)

    override val visible: Boolean
        get() {
            return when {
                // 护盾已耗尽
                !shieldData.hasShield() -> false
                // 实体持续显示护盾
                shieldData.isAlwaysShowShield -> true
                // 经过了太长时间
                shieldData.timeSinceLastHit <= gradientLength -> true
                else -> false
            }
        }

    override fun getColor(partialTick: Float): Color4f {
        val prog = shieldData.eShield / shieldData.maxEShield
        // 满血 (prog = 1) 的时候返回值需要等于 C0
        // dColor = c1 - c0
        // (c1 - c0) * (1 - prog)
        dColor.set(D_COLOR)
        dColor.scale(1F - prog)
        result.set(COLOR0)
        result.add(dColor)
        if (!shieldData.isAlwaysShowShield) {
            result.w *= 1 - (shieldData.timeSinceLastHit + partialTick) * gradientSpeed
            if (result.w < 0) {
                result.w = 0F
            }
        }
        return result
    }
}