package moe.gensoukyo.haloshield.client.model

import net.minecraft.client.model.ModelBase
import net.minecraft.client.model.ModelRenderer
import net.minecraftforge.fml.relauncher.Side
import net.minecraftforge.fml.relauncher.SideOnly
import kotlin.math.round
import kotlin.random.Random

/**
 * 和普通的 [ModelRenderer] 并没有什么两样
 * @author ChloePrime
 */
@SideOnly(Side.CLIENT)
open class EnergyShieldPart(
    shieldModel: ModelBase,
    private val delegate: ModelRenderer
): ModelRenderer(shieldModel, 0, 0) {
    companion object {
        const val shieldSize = 1
    }
    private val antiZFightScale = 1.001f + 0.002f * Random.nextFloat()
    init {
        // 生成盒子外壳是两种护盾模型都要做的
        delegate.cubeList.forEach {
            addBox(
                it.posX1 - shieldSize / 2.0f,
                it.posY1 - shieldSize / 2.0f,
                it.posZ1 - shieldSize / 2.0f,
                round(it.posX2 - it.posX1).toInt() + shieldSize,
                round(it.posY2 - it.posY1).toInt() + shieldSize,
                round(it.posZ2 - it.posZ1).toInt() + shieldSize
            )
        }
        // 应对原部件在构造函数中经过变化的情况
        // 需要在构造时同步模型数据
        syncModelData()
    }

    override fun render(scale: Float) {
        super.render(scale * antiZFightScale)
    }

    protected fun syncModelData() {
        rotationPointX = delegate.rotationPointX
        rotationPointY = delegate.rotationPointY
        rotationPointZ = delegate.rotationPointZ
        rotateAngleX = delegate.rotateAngleX
        rotateAngleY = delegate.rotateAngleY
        rotateAngleZ = delegate.rotateAngleZ
        mirror = delegate.mirror
        showModel = delegate.showModel
        isHidden = delegate.isHidden
    }
}