package moe.gensoukyo.haloshield.client.model

import moe.gensoukyo.haloshield.SpartanShieldsMod
import moe.gensoukyo.haloshield.client.model.internal.IntrusiveEShieldPart
import net.minecraft.client.model.ModelBase
import net.minecraft.client.model.ModelEnderCrystal
import net.minecraft.client.model.ModelRenderer
import net.minecraft.client.model.TextureOffset
import net.minecraft.entity.Entity
import net.minecraft.entity.EntityLivingBase
import net.minecraftforge.fml.relauncher.Side
import net.minecraftforge.fml.relauncher.SideOnly
import org.apache.commons.lang3.reflect.FieldUtils
import java.util.*

/**
 * 侵入原模型并替换 part 的护盾模型，
 * 理论上对某些实体（例如 [ModelEnderCrystal]）的兼容性最好
 */
@SideOnly(Side.CLIENT)
@Suppress("UNCHECKED_CAST")
class IntrusiveEShieldModel(private val origModel: ModelBase): ModelBase() {
    init {
        val fields = ModelPartFinder.getFields(origModel.javaClass)
        val estimatedFieldNum = fields.plainFields.size + fields.arrayFields.size * 2

        // 替换子模型
        val generatedParts = LinkedList<IntrusiveEShieldPart>()
        // 记录模型替换前->后的引用的映射
        val refMap = IdentityHashMap<ModelRenderer, ModelRenderer>(estimatedFieldNum)
        // 移植字段
        fields.plainFields.forEach { field ->
            val origPart = field.get(origModel) as ModelRenderer
            val shieldPart = IntrusiveEShieldPart(this, origPart)
            FieldUtils.removeFinalModifier(field)
            field.set(origModel, shieldPart)

            generatedParts.add(shieldPart)
            refMap[origPart] = shieldPart
        }
        fields.arrayFields.forEach { field ->
            val origParts = field.get(origModel) as Array<ModelRenderer>
            FieldUtils.removeFinalModifier(field)
            val shieldParts = origParts.map { IntrusiveEShieldPart(this, it) }.toTypedArray()
            field.set(origModel, shieldParts)

            generatedParts.addAll(shieldParts)
            origParts.forEachIndexed { index, origPart ->
                val shieldPart = shieldParts[index]
                refMap[origPart] = shieldPart
            }
        }

        // 替换子模型
        generatedParts.forEach { it.replaceChildren(refMap) }
    }

    private var error = false

    override fun render(
        entityIn: Entity,
        limbSwing: Float,
        limbSwingAmount: Float,
        ageInTicks: Float,
        netHeadYaw: Float,
        headPitch: Float,
        scale: Float
    ) {
        if (error) return
        try {
            origModel.render(entityIn, limbSwing, limbSwingAmount, ageInTicks, netHeadYaw, headPitch, scale)
        } catch (e: Exception) {
            SpartanShieldsMod.logger.warn(
                "Error in rendering ${javaClass.canonicalName}, ignoring", e)
            error = true
        }
    }

    override fun setRotationAngles(
        limbSwing: Float,
        limbSwingAmount: Float,
        ageInTicks: Float,
        netHeadYaw: Float,
        headPitch: Float,
        scaleFactor: Float,
        entityIn: Entity
    ) {
        origModel.setRotationAngles(limbSwing, limbSwingAmount, ageInTicks, netHeadYaw, headPitch, scaleFactor, entityIn)
        super.setRotationAngles(limbSwing, limbSwingAmount, ageInTicks, netHeadYaw, headPitch, scaleFactor, entityIn)
    }

    // Bridge Methods

    override fun setLivingAnimations(
        entitylivingbaseIn: EntityLivingBase,
        limbSwing: Float,
        limbSwingAmount: Float,
        partialTickTime: Float
    ) {
        origModel.setLivingAnimations(entitylivingbaseIn, limbSwing, limbSwingAmount, partialTickTime)
        super.setLivingAnimations(entitylivingbaseIn, limbSwing, limbSwingAmount, partialTickTime)
    }

    override fun getTextureOffset(partName: String): TextureOffset {
        return origModel.getTextureOffset(partName)
    }

    override fun setModelAttributes(model: ModelBase) {
        origModel.setModelAttributes(model)
        super.setModelAttributes(model)
    }
}
