package moe.gensoukyo.haloshield.client.model.internal

import moe.gensoukyo.haloshield.SpartanShieldsMod
import moe.gensoukyo.haloshield.client.caps.IllegalModelException
import moe.gensoukyo.haloshield.client.event.GenerateShieldRendererEvent
import moe.gensoukyo.haloshield.client.model.EnergyShieldModel
import moe.gensoukyo.haloshield.client.renderer.RenderEnergyShield
import net.minecraft.client.Minecraft
import net.minecraft.client.renderer.entity.RenderLivingBase
import net.minecraft.entity.EntityLivingBase
import net.minecraftforge.common.MinecraftForge
import net.minecraftforge.fml.relauncher.Side
import net.minecraftforge.fml.relauncher.SideOnly

/**
 * EntityLivingBase -> RenderEnergyShield
 */
@SideOnly(Side.CLIENT)
object GeneratedRenderers: ClassValue<RenderEnergyShield<out EntityLivingBase>>() {
    private val bound = EntityLivingBase::class.java
    /**
     * @throws IllegalModelException 有时 LivingEntity 的模型会不是 RenderLivingBase ，此时报错
     */
    @Suppress("UNCHECKED_CAST")
    fun <E: EntityLivingBase> fromEntity(e: E): RenderEnergyShield<E>
    = get(e.javaClass) as RenderEnergyShield<E>

    @Suppress("UNCHECKED_CAST")
    override fun computeValue(type: Class<*>): RenderEnergyShield<out EntityLivingBase> {
        // 如果有人扔什么乱七八糟的类进来，那么返回一个空模型
        if (!bound.isAssignableFrom(type)) {
            SpartanShieldsMod.logger.warn("Trying to generate shield model for class")
            SpartanShieldsMod.logger.warn("[$type]")
            SpartanShieldsMod.logger.warn("which is not a living entity's class")
            return RenderEnergyShield.EMPTY_RENDERER
        }
        if (type == bound) {
            return RenderEnergyShield.EMPTY_RENDERER
        }
        val entityClass = type as Class<EntityLivingBase>
        val origRenderer = (Minecraft.getMinecraft().renderManager
            .getEntityClassRenderObject<EntityLivingBase>(entityClass)
                as? RenderLivingBase<EntityLivingBase>) ?: throw IllegalModelException()
        val origModel = origRenderer.mainModel

        // 允许event自定义不同的护盾渲染器和模型
        val event = GenerateShieldRendererEvent(entityClass, origRenderer, origModel)
        MinecraftForge.EVENT_BUS.post(event)

        if (event.noModel) {
            return RenderEnergyShield.EMPTY_RENDERER
        }
        return (event.rendererOverride ?: RenderEnergyShield(entityClass,
            (event.modelOverride ?: EnergyShieldModel(origModel))
        ))
    }
}