package moe.gensoukyo.haloshield.client.model.internal

import moe.gensoukyo.haloshield.client.api.IMixinModelRenderer
import moe.gensoukyo.haloshield.client.model.EnergyShieldPart
import net.minecraft.client.model.ModelBase
import net.minecraft.client.model.ModelRenderer
import net.minecraftforge.fml.relauncher.Side
import net.minecraftforge.fml.relauncher.SideOnly

/**
 * 生成式护盾模型专用的护盾部件
 * 会自动向原模型同步数据，生成对应的子模型
 * 并且只有在原部件被渲染后才会渲染
 */
@SideOnly(Side.CLIENT)
class GeneratedShieldPart(
    shieldModel: ModelBase,
    private val delegate: ModelRenderer
): EnergyShieldPart(shieldModel, delegate) {
    init {
        // 生成子模型
        delegate.childModels?.forEach {
            addChild(GeneratedShieldPart(shieldModel, it))
        }
    }
    override fun render(scale: Float) {
        val mixinData = delegate as IMixinModelRenderer
        // 模型本身未渲染时不渲染护盾
        if (!mixinData.shouldRenderShield) {
            return
        }

        syncModelData()
        val origModelScale = mixinData.lastScale
        super.render(origModelScale)

        // 取消应该渲染护盾的标记，待下次模型正常渲染时再次设置为 true
        mixinData.shouldRenderShield = false
    }

}