package moe.gensoukyo.haloshield.common

import moe.gensoukyo.haloshield.SpartanShieldsMod
import moe.gensoukyo.haloshield.common.api.SpartanShieldApi
import net.minecraft.util.DamageSource
import net.minecraftforge.event.entity.living.*
import net.minecraftforge.fml.common.Mod
import net.minecraftforge.fml.common.eventhandler.Event
import net.minecraftforge.fml.common.eventhandler.EventPriority
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent

/**
 * 护盾的额外保护效果。
 * 灭火，消除药水效果等。
 * 此处处理服务端逻辑。
 */
@Mod.EventBusSubscriber(modid = SpartanShieldsMod.MODID)
object ShieldEventHandler {

    /**
     * 护盾自动恢复
     */
    @JvmStatic
    @SubscribeEvent
    fun onTick(e: LivingEvent.LivingUpdateEvent) {
        if (e.entity.world.isRemote) return
        SpartanShieldApi.getShieldData(e.entityLiving).tryRecoverShield()
    }

    /**
     * 抗击退
     */
    @JvmStatic
    @SubscribeEvent
    fun onKnockback(e: LivingKnockBackEvent) {
        if (SpartanShieldApi.getShieldData(e.entityLiving).hasShield()) {
            e.isCanceled = true
        }
    }

    // 受到伤害后重置护盾

    @JvmStatic
    @SubscribeEvent(priority = EventPriority.LOWEST)
    fun onDamageLast(e: LivingDamageEvent) {
        // 这个事件只在逻辑服务端发送
        // 所以此处略去逻辑端判断
        SpartanShieldApi.getShieldData(e.entityLiving).resetRecoverTime()
    }

    // 护盾的保护效果

    @JvmStatic
    @SubscribeEvent
    fun onFireDamage(e: LivingHurtEvent) {
        doProtection(e, e.source, e.amount)
    }
    @JvmStatic
    @SubscribeEvent
    fun onFireDamage(e: LivingAttackEvent) {
        doProtection(e, e.source, e.amount)
    }

    private fun doProtection(e: LivingEvent, source: DamageSource, amount: Float) {
        if (e.entity.world.isRemote) return
        val shieldData = SpartanShieldApi.getShieldData(e.entityLiving)
        if (!shieldData.hasShield()) {
            return
        }

        if (amount < Float.MIN_VALUE) {
            e.isCanceled = true
            return
        }
        // 防火
        if (source.isFireDamage && ModConfig.shieldPreventsFire) {
            e.entity.extinguish()
            e.isCanceled = true
        } else {
            SpartanShieldsMod.logger.debug("Src:$source")
        }
    }

    /**
     * 自动治愈
     */
    @JvmStatic
    @SubscribeEvent
    fun onEffectAdded(e: PotionEvent.PotionApplicableEvent) {
        // 护盾被打空
        if (SpartanShieldApi.getShieldData(e.entityLiving).hasShield().not()) {
            return
        }
        if (e.potionEffect.potion.registryName in ModConfig.autoHealOopList) {
            e.result = Event.Result.DENY
        }
    }
}