package moe.gensoukyo.haloshield.common.api;

import moe.gensoukyo.haloshield.common.ModConfig;

/**
 * 活体实体的能量护盾数据信息
 * @author Administrator
 */
@SuppressWarnings("AlibabaLowerCamelCaseVariableNaming")
public interface IShieldData {
    /**
     * 获得当前护盾值
     *
     * @return 当前护盾值
     */
    float getEShield();

    /**
     * 设置护盾值
     *
     * @param shield 待设置护盾值，会被压缩到 [0, 护盾上限] 中
     */
    void setEShield(float shield);

    /**
     * 获得护盾值上限
     *
     * @return 护盾值上限
     */
    float getMaxEShield();

    /**
     * 设置最大护盾值
     *
     * @param maxShield 目标最大护盾值，最低为0
     */
    void setMaxEShield(float maxShield);

    /**
     * 判断一个实体的护盾是否还有剩余
     * @return 如果护盾上限为0或者护盾被打空，返回false，否则返回true
     */
    default boolean hasShield() {
        return getEShield() > 0;
    }

    /**
     * 尝试恢复护盾
     * 前提条件：
     * 1. 当前时间 >= 最后一次受上时间 + 护盾恢复延迟
     * 2. 护盾未满
     * 此时才会恢复护盾
     */
    void tryRecoverShield();

    /**
     * 重置护盾恢复时间，
     * 重置后需要经过等值于实体护盾恢复速度的tick数，护盾才开始恢复。
     * 在不用的逻辑端调用会产生不同的效果。
     *
     * 在客户端调用时，会刷新护盾闪烁动画的开始时间，
     * 而在服务端调用时，会重置护盾从受伤到开始恢复的时间。
     *
     * @see net.minecraft.world.World#isRemote
     */
    void resetRecoverTime();

    /**
     * 获取距离上一次受上的时间，
     * 会被 {@link IShieldData#resetRecoverTime} 重置。
     * 这个值在客户端和服务端是分别记录的。
     *
     * @see IShieldData#resetRecoverTime
     * @return 当前距离上一次受上的时间
     */
    int getTimeSinceLastHit();

    /**
     * 获取这个实体是否一直显示护盾
     * @return 实体是否一直显示护盾
     * @implSpec 返回的结果在 {@link ModConfig#invertAlwaysShowShield} 时需要和原结果相反
     */
    boolean isAlwaysShowShield();

    /**
     * 设置这个实体是否一直显示护盾
     * @param alwaysShowShield 设置这个实体是否会持续显示护盾渲染
     */
    void setAlwaysShowShield(boolean alwaysShowShield);
}
