package moe.gensoukyo.haloshield.core.common;

import moe.gensoukyo.haloshield.common.ModConfig;
import moe.gensoukyo.haloshield.common.api.IShieldData;
import moe.gensoukyo.haloshield.common.api.SpartanShieldAttributes;
import moe.gensoukyo.haloshield.common.audio.ModSounds;
import moe.gensoukyo.haloshield.common.event.ShieldBreakEvent;
import moe.gensoukyo.haloshield.common.event.ShieldRecoverEvent;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.ai.attributes.AbstractAttributeMap;
import net.minecraft.entity.ai.attributes.IAttribute;
import net.minecraft.entity.ai.attributes.IAttributeInstance;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.datasync.DataParameter;
import net.minecraft.network.datasync.DataSerializers;
import net.minecraft.network.datasync.EntityDataManager;
import net.minecraft.util.DamageSource;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.math.MathHelper;
import net.minecraft.world.World;
import net.minecraftforge.common.MinecraftForge;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.Redirect;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import javax.annotation.Nullable;

/**
 * @author ChloePrime
 */
@SuppressWarnings({
        "AlibabaLowerCamelCaseVariableNaming",
        "AlibabaClassNamingShouldBeCamel",
        "AlibabaAbstractMethodOrInterfaceMethodMustUseJavadoc",
        "AlibabaAbstractClassShouldStartWithAbstractNaming"
})
@Mixin(EntityLivingBase.class)
public abstract class MixinELB extends Entity implements IShieldData {
    @Shadow
    public abstract float getHealth();

    @Shadow
    public abstract IAttributeInstance getEntityAttribute(IAttribute attribute);

    @Shadow
    public abstract void setHealth(float health);

    @Shadow
    public abstract AbstractAttributeMap getAttributeMap();

    @Shadow
    @Nullable
    protected abstract SoundEvent getHurtSound(DamageSource damageSourceIn);
    /**
     * 护盾值属性
     */
    private static final DataParameter<Float> SHIELD
            = EntityDataManager.createKey(EntityLivingBase.class, DataSerializers.FLOAT);
    private static final DataParameter<Boolean> ALWAYS_SHOW_SHIELD
            = EntityDataManager.createKey(EntityLivingBase.class, DataSerializers.BOOLEAN);
    /**
     * NBT标签名称
     */
    private static final String TAG_SHIELD = "EShield";
    private static final String TAG_INVERT_SHIELD_VISIBILITY = "es_always_vis";

    public MixinELB(World worldIn) {
        super(worldIn);
        throw new InternalError("Invoking a constructor of a mixin class");
    }

    private EntityLivingBase cast() {
        return  (EntityLivingBase) (Object) this;
    }

    @Override
    public float getEShield() {
        return dataManager.get(SHIELD);
    }

    @Override
    public void setEShield(float shield) {
        dataManager.set(SHIELD, MathHelper.clamp(shield, 0F, getMaxEShield()));
    }

    @Override
    public float getMaxEShield() {
        return (float) getEntityAttribute(SpartanShieldAttributes.MAX_SHIELD_ATTR).getAttributeValue();
    }

    @Override
    public void setMaxEShield(float maxShield) {
        getEntityAttribute(SpartanShieldAttributes.MAX_SHIELD_ATTR).setBaseValue(maxShield);
    }

    @Override
    public boolean isAlwaysShowShield() {
        return getAlwaysShowShieldFlag() ^ ModConfig.invertAlwaysShowShield;
    }

    private boolean getAlwaysShowShieldFlag() {
        return dataManager.get(ALWAYS_SHOW_SHIELD);
    }

    @Override
    public void setAlwaysShowShield(boolean alwaysShowShield) {
        dataManager.set(ALWAYS_SHOW_SHIELD, alwaysShowShield);
    }

    @Override
    public void tryRecoverShield() {
        double delay = getEntityAttribute(SpartanShieldAttributes.SHIELD_RECOVER_DELAY).getAttributeValue();
        if (getTimeSinceLastHit() < delay) {
            return;
        }
        double amount = getEntityAttribute(SpartanShieldAttributes.SHIELD_RECOVER_SPEED).getAttributeValue();
        if (!hasShield()) {
            ShieldRecoverEvent event = new ShieldRecoverEvent(cast());
            MinecraftForge.EVENT_BUS.post(event);
        }
        if (getEShield() < getMaxEShield()) {
            setEShield(getEShield() + ((float) amount));
        }
    }

    private int lastTimeHit;
    @Override
    public void resetRecoverTime() {
        lastTimeHit = ticksExisted;
    }

    @Override
    public int getTimeSinceLastHit() {
        return ticksExisted - lastTimeHit;
    }

    // 护盾的实际效果

    @Inject(
            method = "setHealth(F)V",
            at = @At(
                    value = "HEAD"
            ),
            cancellable = true
    )
    private void inject_setHealth(float health, CallbackInfo ci) {
        if (stopInject) {
            return;
        }
        float shield = getEShield();
        // 无护盾或护盾已被打完，直接返回
        // 交由原版代码处理
        if (shield <= 0) {
            return;
        }
        float damage = getHealth() - health;
        // 回血的情况，不处理护盾
        if (damage <= 0) {
            return;
        }
        damageShield(shield, damage);
        ci.cancel();
    }

    private void damageShield(float current, float damage) {
        current -= damage;
        setEShield(current);
        // 伤害超出护盾值的情况
        float realDamage = -current;
        if (realDamage >= 0) {
            postDamagedEvent();
        }
        if (realDamage > 0) {
            stopInject = true;
            setHealth(getHealth() - realDamage);
            stopInject = false;
        }
    }

    private void postDamagedEvent() {
        ShieldBreakEvent event = new ShieldBreakEvent(cast(), cast().getLastDamageSource());
        MinecraftForge.EVENT_BUS.post(event);
    }

    private boolean stopInject;

    /**
     * 修改音效
     *
     * @return 有护盾时返回护盾受损音效，否则返回原音效
     */
    @Redirect(
            method = {"handleStatusUpdate(B)V", "playHurtSound(Lnet/minecraft/util/DamageSource;)V"},
            at = @At(
                    value = "INVOKE",
                    target = "Lnet/minecraft/entity/EntityLivingBase;getHurtSound(Lnet/minecraft/util/DamageSource;)Lnet/minecraft/util/SoundEvent;"
            )
    )
    @SuppressWarnings("ConstantConditions")
    private SoundEvent redirect_getHurtSound(EntityLivingBase thiz, DamageSource damageSrc) {
        MixinELB casted = (MixinELB) (Object) thiz;
        if (casted.getEShield() > 0) {
            return ModSounds.SND_SHIELD_DAMAGE;
        }
        return casted.getHurtSound(damageSrc);
    }

    // 数据注册，持久化

    @Inject(
            method = "entityInit()V",
            at = @At("TAIL")
    )
    private void inject_entityInit(CallbackInfo ci) {
        dataManager.register(SHIELD, 0F);
        dataManager.register(ALWAYS_SHOW_SHIELD, false);
    }

    @Inject(
            method = "applyEntityAttributes()V",
            at = @At("TAIL")
    )
    private void inject_applyEntityAttributes(CallbackInfo ci) {
        getAttributeMap().registerAttribute(SpartanShieldAttributes.MAX_SHIELD_ATTR);
        getAttributeMap().registerAttribute(SpartanShieldAttributes.SHIELD_RECOVER_SPEED);
        getAttributeMap().registerAttribute(SpartanShieldAttributes.SHIELD_RECOVER_DELAY);
        setEShield(getMaxEShield());
    }

    @Inject(
            method = "readEntityFromNBT(Lnet/minecraft/nbt/NBTTagCompound;)V",
            at = @At("TAIL")
    )
    private void inject_readEntityFromNBT(NBTTagCompound compound, CallbackInfo ci) {
        setEShield(compound.getFloat(TAG_SHIELD));
        setAlwaysShowShield(compound.getBoolean(TAG_INVERT_SHIELD_VISIBILITY));
    }

    @Inject(
            method = "writeEntityToNBT(Lnet/minecraft/nbt/NBTTagCompound;)V",
            at = @At("TAIL")
    )
    private void inject_writeEntityToNBT(NBTTagCompound compound, CallbackInfo ci) {
        compound.setFloat(TAG_SHIELD, getEShield());
        compound.setBoolean(TAG_INVERT_SHIELD_VISIBILITY, getAlwaysShowShieldFlag());
    }
}
