package moe.gensoukyo.scriptap;

import net.minecraft.entity.Entity;
import net.minecraft.server.MinecraftServer;
import net.minecraft.world.WorldServer;
import noppes.npcs.api.NpcAPI;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import java.util.regex.Pattern;


import java.lang.invoke.MethodHandle;
import java.lang.invoke.MethodHandles;
import java.lang.invoke.MethodType;
import java.util.regex.Pattern;

/**
 * {@link org.bukkit.entity.Entity}
 * to
 * {@link net.minecraft.entity.Entity}
 * 快速的转换需要CraftBukkit的实现中有getHandle函数
 * @author Chloe_koopa
 */
public class EntityTransformer {
    private static MethodHandle TRANSFORMER = null;
    private static boolean hasNativeTransformer = true;
    private static final Logger LOGGER = LogManager
            .getLogger("ScriptAp Native Entity Transformer");

    private static final Pattern ENTITY_CLASS_PATTERN =
            Pattern.compile("org\\.bukkit\\.craftbukkit.*\\.entity\\.CraftEntity");

    private static void setupNativeTransformer(org.bukkit.entity.Entity bukkitEntity) {
        Class<?> curClass = bukkitEntity.getClass();
        while (curClass != null) {
            //寻找CraftEntity对应的类
            if (ENTITY_CLASS_PATTERN.matcher(curClass.getName()).matches()) {
                break;
            }
            curClass = curClass.getSuperclass();
        }
        if (curClass == null) {
            hasNativeTransformer = false;
            return;
        }

        try {
            TRANSFORMER = MethodHandles.lookup().findVirtual(
                    curClass,
                    "getHandle",
                    MethodType.methodType(Entity.class)
            );
        } catch (Exception e) {
            LOGGER.error("Can't setup native Entity transformer!", e);
            hasNativeTransformer = false;
        }
    }

    public static Entity transform(org.bukkit.entity.Entity craftEntity) {
        if ((TRANSFORMER == null) && hasNativeTransformer) {
            setupNativeTransformer(craftEntity);
        }
        if (hasNativeTransformer) {
            try {
                return (Entity) TRANSFORMER.bindTo(craftEntity).invoke();
            } catch (Throwable t) {
                LOGGER.error("What happened ?", t);
                hasNativeTransformer = false;
            }
        }
        return transformOld(craftEntity);
    }

    private static final MinecraftServer SERVER =
            NpcAPI.Instance().getIWorld(0).getMCWorld().func_73046_m();
    static Entity transformOld(org.bukkit.entity.Entity bukkitSide) {
        assert SERVER != null;
        int id = bukkitSide.getEntityId();
        Entity result = null;
        for (WorldServer world : SERVER.field_71305_c) {
            result = world.func_73045_a(id);
            if (result != null) {
                break;
            }
        }
        return result;
    }
}
